package com.nisum.ascend.promotions.exception;

import org.springframework.core.annotation.Order;
import org.springframework.core.io.buffer.DataBuffer;

import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import org.springframework.web.server.WebExceptionHandler;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;

@Component
@Order(-2)
class RestWebExceptionHandler implements WebExceptionHandler {

    @Override
    public Mono<Void> handle(ServerWebExchange exchange, Throwable ex) {
        if (ex instanceof PromotionNotFoundException) {
            exchange.getResponse().setStatusCode(HttpStatus.NOT_FOUND);
            DataBuffer buffer = exchange.getResponse().bufferFactory().wrap(ex.getMessage().getBytes(StandardCharsets.UTF_8));
            // marks the response as complete and forbids writing to it
            return exchange.getResponse().writeWith(Flux.just(buffer));
        }
        if (ex instanceof PromotionAlreadyExistsException) {
            exchange.getResponse().setStatusCode(HttpStatus.NOT_ACCEPTABLE);

            return exchange.getResponse().setComplete();
        }
        return Mono.error(ex);
    }
}
